//   ==================================================================
//   >>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
//   ------------------------------------------------------------------
//   Copyright (c) 2006-2011 by Lattice Semiconductor Corporation
//   ALL RIGHTS RESERVED 
//   ------------------------------------------------------------------
//
//   IMPORTANT: THIS FILE IS AUTO-GENERATED BY THE LATTICEMICO SYSTEM.
//
//   Permission:
//
//      Lattice Semiconductor grants permission to use this code
//      pursuant to the terms of the Lattice Semiconductor Corporation
//      Open Source License Agreement.  
//
//   Disclaimer:
//
//      Lattice Semiconductor provides no warranty regarding the use or
//      functionality of this code. It is the user's responsibility to
//      verify the users design for consistency and functionality through
//      the use of formal verification methods.
//
//   --------------------------------------------------------------------
//
//                  Lattice Semiconductor Corporation
//                  5555 NE Moore Court
//                  Hillsboro, OR 97214
//                  U.S.A
//
//                  TEL: 1-800-Lattice (USA and Canada)
//                         503-286-8001 (other locations)
//
//                  web: http://www.latticesemi.com/
//                  email: techsupport@latticesemi.com
//
//   --------------------------------------------------------------------
//                         FILE DETAILS
// Project          : SDRAM Controller
// File             : sdr_ctrl.v
// Title            : SDRAM Control logic 
// Dependencies     : sdr_par.v
// Revision History : 1.0: Initial Release
//                    3.0: Support for burst reads
//                    3.1: Support for burst writes
//                    3.2: (1) PLL used for SDRAM clock generation
//                         (2) IO registers are used for all SDRAM interface signals
//                    3.3: (1) PMI PLL updated to use correct primitive
//                         (2) Fix precharge bug for reads/writes
//                    3.4: (1) Fix for overflow bug for 32-bit burst reads
//                         (2) Fix for column addressing bug for 32-bit reads/writes
// =============================================================================


module sdr_ctrl(
  //global
  sys_CLK,
  sys_RESET,
  //fifo interface
  wb2sdr_re,
  wb2sdr_q,
  wb2sdr_empty,
  wb2sdr_almost_empty,
  sdr2wb_full,
  sdr2wb_almost_full,
  //System interface
  sys_DLY_100US,
  iState,
  cState,
  cState_d,
  clkCNT,
  S_STB_I,
  read_tmn,
  rd_cmd,
  rd_cmd_cnt,
  cross_page,
  write_done
);
`include "sdr_par.v"

input                                   sys_CLK;
input                                   sys_RESET;
input [68:0]                            wb2sdr_q;
input                                   wb2sdr_empty;
input                                   wb2sdr_almost_empty;
input                                   sdr2wb_full;
input                                   sdr2wb_almost_full;
input                                   sys_DLY_100US;
input                                   S_STB_I;
input                                   cross_page;
output                                  wb2sdr_re;
output [3:0]                            iState;
output [3:0]                            cState;
output [3:0]                            cState_d;
output [3:0]                            clkCNT;
output                                  read_tmn;
output                                  rd_cmd;
output [`SDR_COL_WIDTH+`SDR_BNK_WDTH:0] rd_cmd_cnt;
output                                  write_done;

//---------------------------------------------------------------------
// registers
//
wire                                 wb2sdr_re;
wire                                 rd_cmd;
reg                                  sys_INIT_DONE;   // indicates sdr initialization is done
reg [3:0]                            iState;          // INIT_FSM state variables
reg [3:0]                            cState;          // CMD_FSM state variables
reg [3:0]                            cState_d;
reg [3:0]                            clkCNT;
reg                                  syncResetClkCNT; // reset clkCNT to 0
reg                                  sys_REF_REQ;
reg [11:0]                           ar_cnt;
reg                                  read_tmn;
reg [`SDR_COL_WIDTH+`SDR_BNK_WDTH:0] rd_cmd_cnt;
reg                                  write_done;

//---------------------------------------------------------------------
// local definitions
//
`define endOf_tRP          clkCNT == `SDR_TRP-1
`define endOf_tRFC         clkCNT == `SDR_TRFC-1
`define endOf_tMRD         clkCNT == `SDR_TMRD-1
`define endOf_tRCD         clkCNT == `SDR_TRCD-1
`define endOf_Cas_Latency  clkCNT == NUM_CLK_CL-1
`define endOf_Read_Burst   clkCNT == NUM_CLK_READ - 1
`define endOf_Write_Burst  clkCNT == NUM_CLK_WRITE-2
`define endOf_tDAL         clkCNT == NUM_CLK_WAIT
`define endOf_Precharge    clkCNT == `SDR_TRP-2

always @(posedge sys_CLK or posedge sys_RESET)
   if (sys_RESET) begin
      write_done <= 1'b0;
   end else begin
      if (wb2sdr_empty & (cState == c_idle))
         write_done <= 1'b1;
      else if (!wb2sdr_empty)
         write_done <= 1'b0;
   end
   
//---------------------------------------------------------------------
// auto refresh require
//
always @(posedge sys_CLK or posedge sys_RESET)
   if (sys_RESET) begin
      sys_REF_REQ <= 1'b0;
      ar_cnt      <= `SDR_TREFI;
   end else begin
     if(ar_cnt == 12'd0)
        ar_cnt <= `SDR_TREFI;
     else
        ar_cnt <= ar_cnt - 1;

     if (cState == c_AR)
        sys_REF_REQ <= 1'b0;
     else if(ar_cnt == 'b0)
        sys_REF_REQ <= 1'b1;
   end
   
//---------------------------------------------------------------------
// INIT_FSM state machine
//
always @(posedge sys_CLK or posedge sys_RESET)
  if (sys_RESET) begin
     iState <= #tDLY i_NOP;
  end else
    case (iState)
      i_NOP:   // wait for 100 us delay by checking sys_DLY_100US
               if (sys_DLY_100US) iState <= #tDLY i_PRE;
      i_PRE:   // precharge all
               iState <= #tDLY (`SDR_TRP == 0) ? i_AR1 : i_tRP;
      i_tRP:   // wait until tRP satisfied
               if (`endOf_tRP) iState <= #tDLY i_AR1;
      i_AR1:   // auto referesh
               iState <= #tDLY (`SDR_TRFC == 0) ? i_AR2 : i_tRFC1;
      i_tRFC1: // wait until tRFC satisfied
               if (`endOf_tRFC) iState <= #tDLY i_AR2;
      i_AR2:   // auto referesh
               iState <= #tDLY (`SDR_TRFC == 0) ? i_MRS : i_tRFC2;
      i_tRFC2: // wait until tRFC satisfied
               if (`endOf_tRFC) iState <= #tDLY i_MRS;
      i_MRS:   // load mode register
               iState <= #tDLY (`SDR_TMRD == 0) ? i_ready : i_tMRD;
      i_tMRD:  // wait until tMRD satisfied
               if (`endOf_tMRD) iState <= #tDLY i_ready;
      i_ready: // stay at this state for normal operation
               iState <= #tDLY i_ready;
      default:
               iState <= #tDLY i_NOP;
    endcase

//
// sys_INIT_DONE generation
//
always @(posedge sys_CLK or posedge sys_RESET)
  if (sys_RESET) begin
     sys_INIT_DONE <= #tDLY 0;
  end else
    case (iState)
      i_ready: sys_INIT_DONE <= #tDLY 1;
      default: sys_INIT_DONE <= #tDLY 0;
    endcase

//---------------------------------------------------------------------
// CMD_FSM state machine
//
always @(posedge sys_CLK or posedge sys_RESET)
   if (sys_RESET) begin
      rd_cmd_cnt <= #tDLY 0;
      read_tmn   <= #tDLY 1'b0;
   end else begin
      if ((`endOf_Read_Burst) && (cState == c_rdata))
         rd_cmd_cnt <= #tDLY 0;
      else if (cState == c_READA || cState == c_cl || cState == c_rdata)
         rd_cmd_cnt <= #tDLY rd_cmd_cnt+1;
      else
         rd_cmd_cnt <= #tDLY 0;

      if (~S_STB_I)
         read_tmn <= 1'b1;
      else if (cState == c_ACTIVE)
         read_tmn <= 1'b0;
   end


`ifdef SDR_DATA_SIZE_32
assign rd_cmd = 1'b1;
`endif
`ifdef SDR_DATA_SIZE_16
assign rd_cmd = (rd_cmd_cnt[0] == 1'b0) ? 1'b1 : 1'b0;
`endif
`ifdef SDR_DATA_SIZE_8
assign rd_cmd = (rd_cmd_cnt[1:0] == 2'b00) ? 1'b1 : 1'b0;
`endif
`ifdef SDR_DATA_SIZE_4
assign rd_cmd = (rd_cmd_cnt[2:0] == 3'b000) ? 1'b1 : 1'b0;
`endif

always @(posedge sys_CLK or posedge sys_RESET)
  if (sys_RESET)
     cState_d <= #tDLY c_idle;
  else
     cState_d <= #tDLY cState;

always @(posedge sys_CLK or posedge sys_RESET)
  if (sys_RESET) begin
     cState <= #tDLY c_idle;
  end else
    case (cState)
      c_idle:   // wait until refresh request or addr strobe asserted
                if (sys_REF_REQ && sys_INIT_DONE) 
                    cState <= #tDLY c_AR;
                else if (!wb2sdr_empty && sys_INIT_DONE) 
                    cState <= #tDLY c_ReWait;
      c_ReWait: if (!sdr2wb_full) 
                    cState <= c_ACTIVE;
      c_ACTIVE: // assert row/bank addr
                if (`SDR_TRCD == 1)
                    cState <= #tDLY (wb2sdr_q[68]) ? c_WRITEA : c_READA;
                else 
                    cState <= #tDLY c_tRCD;
      c_tRCD:   // wait until tRCD satisfied
                if (`endOf_tRCD)
                    cState <= #tDLY (wb2sdr_q[68]) ? c_WRITEA : c_READA;
      c_READA:  // assert col/bank addr for read with auto-precharge
                if ((rd_cmd & (sdr2wb_almost_full | cross_page)) | read_tmn)
                    cState <= #tDLY c_cl;
      c_cl:     // CASn latency
                if (`endOf_Cas_Latency)
                    cState <= #tDLY c_rdata;
      c_rdata:  // read cycle data phase
                if (`endOf_Read_Burst) 
                   cState <= #tDLY c_PRECH;
      c_PRECH:  // precharege
                 cState <= #tDLY c_tRP;
      c_tRP:    // tRP latency
                if (`endOf_Precharge)
                    cState <= #tDLY (read_tmn | wb2sdr_q[68]) ? c_idle :
                                    sdr2wb_full ? c_ReWait : c_ACTIVE;
      c_WRITEA: // assert col/bank addr for write with auto-precharge
                if (NUM_CLK_WRITE > 1)
                   cState <= #tDLY c_wdata;
                else if (wb2sdr_almost_empty || cross_page) begin
                   if (`SDR_TDAL < 4)
                      cState <= #tDLY c_PRECH;
                   else
                      cState <= #tDLY c_tDAL;
                end
      c_wdata:  // write cycle data phase
                if (`endOf_Write_Burst) begin
                   if (wb2sdr_almost_empty | cross_page) begin
                      if (`SDR_TDAL < 4)
                         cState <= #tDLY c_PRECH;
                      else
                         cState <= #tDLY c_tDAL;
                   end else
                      cState <= #tDLY c_WRITEA;
                end
      c_tDAL:   // wait until (tWR + tRP) satisfied before issuing next
                // SDRAM ACTIVE command
                if (`endOf_tDAL) 
                    cState <= #tDLY c_PRECH;
      c_AR:     // auto-refresh
                cState <= #tDLY (`SDR_TRFC == 1) ? c_idle : c_tRFC;
      c_tRFC:   // wait until tRFC satisfied
                if (`endOf_tRFC) 
                    cState <= #tDLY c_idle;
      default:
                cState <= #tDLY c_idle;
    endcase

//wb2sdr_re generation
assign wb2sdr_re = (cState == c_idle) && !sys_REF_REQ && !wb2sdr_empty && sys_INIT_DONE ||
                   (cState == c_WRITEA) && (NUM_CLK_WRITE == 1) && !wb2sdr_almost_empty && !cross_page ||
                   (cState == c_wdata) && (NUM_CLK_WRITE > 1) && `endOf_Write_Burst && !wb2sdr_almost_empty && !cross_page;
//---------------------------------------------------------------------
// Clock Counter
//
always @(posedge sys_CLK)
  if (syncResetClkCNT) clkCNT <= #tDLY 0;
  else clkCNT <= #tDLY clkCNT + 1;

//
// syncResetClkCNT generation
//
always @(iState or cState or clkCNT)
  case (iState)
    i_PRE:
         syncResetClkCNT <= #tDLY (`SDR_TRP == 1) ? 1 : 0;
    i_AR1,
    i_AR2:
         syncResetClkCNT <= #tDLY (`SDR_TRFC == 1) ? 1 : 0;
    i_NOP:
         syncResetClkCNT <= #tDLY 1;
    i_tRP:
         syncResetClkCNT <= #tDLY (`endOf_tRP) ? 1 : 0;
    i_tMRD:
         syncResetClkCNT <= #tDLY (`endOf_tMRD) ? 1 : 0;
    i_tRFC1,
    i_tRFC2:
         syncResetClkCNT <= #tDLY (`endOf_tRFC) ? 1 : 0;
    i_ready:
         case (cState)
           c_ACTIVE:
                syncResetClkCNT <= #tDLY (`SDR_TRCD == 1) ? 1 : 0;
           c_idle:
                syncResetClkCNT <= #tDLY 1;
           c_tRCD:
                syncResetClkCNT <= #tDLY (`endOf_tRCD) ? 1 : 0;
           c_tRFC:
                syncResetClkCNT <= #tDLY (`endOf_tRFC) ? 1 : 0;
           c_cl:
                syncResetClkCNT <= #tDLY (`endOf_Cas_Latency) ? 1 : 0;
           c_rdata:
                syncResetClkCNT <= #tDLY (clkCNT == NUM_CLK_READ-1) ? 1 : 0;
           c_tRP:
                syncResetClkCNT <= #tDLY (`endOf_Precharge) ? 1 : 0;
           c_wdata:
                syncResetClkCNT <= #tDLY (`endOf_Write_Burst) ? 1 : 0;
           c_tDAL:
                syncResetClkCNT <= #tDLY (`endOf_tDAL) ? 1 : 0; 
           default:
                syncResetClkCNT <= #tDLY 1;
         endcase
    default:
         syncResetClkCNT <= #tDLY 0;
  endcase

endmodule






