//   ==================================================================
//   >>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
//   ------------------------------------------------------------------
//   Copyright (c) 2006-2011 by Lattice Semiconductor Corporation
//   ALL RIGHTS RESERVED 
//   ------------------------------------------------------------------
//
//   IMPORTANT: THIS FILE IS AUTO-GENERATED BY THE LATTICEMICO SYSTEM.
//
//   Permission:
//
//      Lattice Semiconductor grants permission to use this code
//      pursuant to the terms of the Lattice Semiconductor Corporation
//      Open Source License Agreement.  
//
//   Disclaimer:
//
//      Lattice Semiconductor provides no warranty regarding the use or
//      functionality of this code. It is the user's responsibility to
//      verify the users design for consistency and functionality through
//      the use of formal verification methods.
//
//   --------------------------------------------------------------------
//
//                  Lattice Semiconductor Corporation
//                  5555 NE Moore Court
//                  Hillsboro, OR 97214
//                  U.S.A
//
//                  TEL: 1-800-Lattice (USA and Canada)
//                         503-286-8001 (other locations)
//
//                  web: http://www.latticesemi.com/
//                  email: techsupport@latticesemi.com
//
//   --------------------------------------------------------------------
//                         FILE DETAILS
// Project          : SDRAM Controller
// File             : sdr_data.v
// Title            : SDRAM data logic
// Dependencies     : sdr_par.v
// Revision History : 1.0: Initial Release
//                    3.0: Support for burst reads
//                    3.1: Support for burst writes
//                    3.2: (1) PLL used for SDRAM clock generation
//                         (2) IO registers are used for all SDRAM interface signals
//                    3.3: (1) PMI PLL updated to use correct primitive
//                         (2) Fix precharge bug for reads/writes
//                    3.4: (1) Fix for overflow bug for 32-bit burst reads
//                         (2) Fix for column addressing bug for 32-bit reads/writes
// =============================================================================


//This is the data module for a synchronous DRAM controller.

module sdr_data(
  sys_CLK,
  sys_RESET,
  wb2sdr_q,    // data bus
  sdr2wb_we,
  sdr2wb_data,
  sdr2wb_full,
  cState,
  clkCNT,
  rd_cmd_cnt,
  sdr_DQM,
  sdr_DQ       // sdr data
);

`include "sdr_par.v"

//---------------------------------------------------------------------
// inputs
//
input                                  sys_CLK;
input                                  sys_RESET;
input  [68:0]                          wb2sdr_q;
input                                  sdr2wb_full;
input  [3:0]                           cState;
input  [3:0]                           clkCNT;
input [`SDR_COL_WIDTH+`SDR_BNK_WDTH:0] rd_cmd_cnt;
output                                 sdr2wb_we;
output [31:0]                          sdr2wb_data;
output [`SDR_USER_DM-1:0]              sdr_DQM;
inout  [`SDR_DATA_WIDTH-1:0]           sdr_DQ;

//wires & regs
wire                                 sdr_dat_valid;
wire [`SDR_DATA_WIDTH-1:0]           sdr_DQ_in;
wire [31:0]                          regSysD;
wire [3:0]                           regSEL;
reg                                  sdr2wb_we;
reg [`SDR_COL_WIDTH+`SDR_BNK_WDTH:0] rd_dat_cnt;
reg  [`SDR_DATA_WIDTH-1:0]           sdr_DQ_in_r/*synthesis useioff=1 */;
reg  [`SDR_USER_DM-1:0]              sdr_DQM    /* synthesis syn_preserve=1 *//*synthesis useioff=1 */;
reg [31:0]                           regSdrDQ;
reg                                  enableSysD;
reg [`SDR_DATA_WIDTH-1:0]            regSysDX   /* synthesis syn_preserve=1 *//*synthesis useioff=1 */;
reg [`SDR_DATA_WIDTH-1:0]            enableSdrDQ/* synthesis syn_preserve=1 *//*synthesis useioff=1 */;

`ifdef SDR_DATA_SIZE_4
wire [3:0]   cnt0_sdrdq ;
wire [3:0]   cnt1_sdrdq ;
wire [3:0]   cnt2_sdrdq ;
wire [3:0]   cnt3_sdrdq ;
wire [3:0]   cnt4_sdrdq ;
wire [3:0]   cnt5_sdrdq ;
wire [3:0]   cnt6_sdrdq ;
wire [3:0]   cnt7_sdrdq ;
`else
  `ifdef SDR_DATA_SIZE_8
wire [7:0]   cnt0_sdrdq ;
wire [7:0]   cnt1_sdrdq ;
wire [7:0]   cnt2_sdrdq ;
wire [7:0]   cnt3_sdrdq ;
  `else
    `ifdef SDR_DATA_SIZE_16
wire [15:0]  cnt0_sdrdq ;
wire [15:0]  cnt1_sdrdq ;
    `else
wire [31:0]  cnt0_sdrdq ;
    `endif
  `endif
`endif


always @(posedge sys_CLK or posedge sys_RESET)
   if (sys_RESET)
      rd_dat_cnt <= #tDLY 0;
   else if (rd_cmd_cnt > NUM_CLK_CL+1)
      rd_dat_cnt <= #tDLY rd_cmd_cnt - NUM_CLK_CL-1;
   else
      rd_dat_cnt <= #tDLY 0;

always @(posedge sys_CLK or posedge sys_RESET)
   if (sys_RESET)
      sdr_DQ_in_r <= #tDLY 0;
   else
      sdr_DQ_in_r <= #tDLY sdr_DQ_in;

`ifdef SDR_DATA_SIZE_4
assign cnt0_sdrdq = (rd_dat_cnt[2:0] == 0) ? sdr_DQ_in_r : regSdrDQ[3:0];
assign cnt1_sdrdq = (rd_dat_cnt[2:0] == 1) ? sdr_DQ_in_r : regSdrDQ[7:4];
assign cnt2_sdrdq = (rd_dat_cnt[2:0] == 2) ? sdr_DQ_in_r : regSdrDQ[11:8];
assign cnt3_sdrdq = (rd_dat_cnt[2:0] == 3) ? sdr_DQ_in_r : regSdrDQ[15:12];
assign cnt4_sdrdq = (rd_dat_cnt[2:0] == 4) ? sdr_DQ_in_r : regSdrDQ[19:16];
assign cnt5_sdrdq = (rd_dat_cnt[2:0] == 5) ? sdr_DQ_in_r : regSdrDQ[23:20];
assign cnt6_sdrdq = (rd_dat_cnt[2:0] == 6) ? sdr_DQ_in_r : regSdrDQ[27:24];
assign cnt7_sdrdq = (rd_dat_cnt[2:0] == 7) ? sdr_DQ_in_r : regSdrDQ[31:28];
assign sdr_dat_valid = (rd_dat_cnt[2:0] == 7) ? 1'b1 : 1'b0;
`else
  `ifdef SDR_DATA_SIZE_8
assign cnt0_sdrdq = (rd_dat_cnt[1:0] == 0) ? sdr_DQ_in_r : regSdrDQ[7:0];
assign cnt1_sdrdq = (rd_dat_cnt[1:0] == 1) ? sdr_DQ_in_r : regSdrDQ[15:8];
assign cnt2_sdrdq = (rd_dat_cnt[1:0] == 2) ? sdr_DQ_in_r : regSdrDQ[23:16];
assign cnt3_sdrdq = (rd_dat_cnt[1:0] == 3) ? sdr_DQ_in_r : regSdrDQ[31:24];
assign sdr_dat_valid = (rd_dat_cnt[1:0] == 3) ? 1'b1 : 1'b0;
  `else
    `ifdef SDR_DATA_SIZE_16
assign cnt0_sdrdq = (rd_dat_cnt[0] == 0) ? sdr_DQ_in_r : regSdrDQ[15:0];
assign cnt1_sdrdq = (rd_dat_cnt[0] == 1) ? sdr_DQ_in_r : regSdrDQ[31:16];
assign sdr_dat_valid = (rd_dat_cnt[0] == 1) ? 1'b1 : 1'b0;
    `else
assign cnt0_sdrdq = sdr_DQ_in_r;
assign sdr_dat_valid = 1'b1;
    `endif
  `endif
`endif

//---------------------------------------------------------------------
//  Read Cycle Data Path
always @(posedge sys_CLK or posedge sys_RESET)
   if (sys_RESET)
      sdr2wb_we <= #tDLY 0;
   else
      sdr2wb_we <= #tDLY sdr_dat_valid && enableSysD;

assign #tDLY sdr2wb_data = regSdrDQ;

always @(posedge sys_CLK or posedge sys_RESET)
   if (sys_RESET)
      regSdrDQ <= #tDLY 0;
   else
     `ifdef SDR_DATA_SIZE_4
       regSdrDQ <= #tDLY {cnt7_sdrdq,cnt6_sdrdq,cnt5_sdrdq,cnt4_sdrdq,cnt3_sdrdq,cnt2_sdrdq,cnt1_sdrdq,cnt0_sdrdq};
     `else
       `ifdef SDR_DATA_SIZE_8
         regSdrDQ <= #tDLY {cnt3_sdrdq,cnt2_sdrdq,cnt1_sdrdq,cnt0_sdrdq};
       `else
         `ifdef SDR_DATA_SIZE_16
           regSdrDQ <= #tDLY {cnt1_sdrdq,cnt0_sdrdq};
         `else
           regSdrDQ <= #tDLY cnt0_sdrdq;
         `endif
       `endif
     `endif

always @(posedge sys_CLK or posedge sys_RESET)
  if (sys_RESET)
          enableSysD <= #tDLY 0;
  else if ((rd_cmd_cnt > NUM_CLK_CL) && (cState == c_READA || cState == c_cl || cState == c_rdata))
          enableSysD <= #tDLY 1;
  else    enableSysD <= #tDLY 0;

//---------------------------------------------------------------------
//  Write Cycle Data Path
//
   generate
   genvar  i;
     for (i = 0 ; i < `SDR_DATA_WIDTH; i = i + 1 )
     begin : u
        BB sdram_data_inst(.I(regSysDX[i]), .T(enableSdrDQ[i]), .O(sdr_DQ_in[i]), .B(sdr_DQ[i]));
     end // block: u
   endgenerate

`ifdef SDR_DATA_SIZE_4
  always @(posedge sys_CLK or posedge sys_RESET)
    if (sys_RESET)
            sdr_DQM <= #tDLY 0;
    else if (cState == c_WRITEA)
            sdr_DQM <= #tDLY regSEL[0];
    else if ((cState == c_wdata) && (clkCNT == 1))
            sdr_DQM <= #tDLY regSEL[1];
    else if ((cState == c_wdata) && (clkCNT == 3))
            sdr_DQM <= #tDLY regSEL[2];
    else if ((cState == c_wdata) && (clkCNT == 5))
            sdr_DQM <= #tDLY regSEL[3];
    else
            sdr_DQM <= #tDLY 0;

  always @(posedge sys_CLK or posedge sys_RESET)
    if (sys_RESET)
            regSysDX <= #tDLY 0;
    else if (cState == c_WRITEA)
            regSysDX <= #tDLY regSysD[3:0];
    else if ((cState == c_wdata) && (clkCNT == 0))
            regSysDX <= #tDLY regSysD[7:4];
    else if ((cState == c_wdata) && (clkCNT == 1))
            regSysDX <= #tDLY regSysD[11:8];
    else if ((cState == c_wdata) && (clkCNT == 2))
            regSysDX <= #tDLY regSysD[15:12];
    else if ((cState == c_wdata) && (clkCNT == 3))
            regSysDX <= #tDLY regSysD[19:16];
    else if ((cState == c_wdata) && (clkCNT == 4))
            regSysDX <= #tDLY regSysD[23:20];
    else if ((cState == c_wdata) && (clkCNT == 5))
            regSysDX <= #tDLY regSysD[27:24];
    else if ((cState == c_wdata) && (clkCNT == 6))
            regSysDX <= #tDLY regSysD[31:28];
    else
            regSysDX <= #tDLY 0;
`else
  `ifdef SDR_DATA_SIZE_8
  always @(posedge sys_CLK or posedge sys_RESET)
    if (sys_RESET)
            sdr_DQM <= #tDLY 0;
    else if (cState == c_WRITEA)
            sdr_DQM <= #tDLY regSEL[0];
    else if ((cState == c_wdata) && (clkCNT == 0))
            sdr_DQM <= #tDLY regSEL[1];
    else if ((cState == c_wdata) && (clkCNT == 1))
            sdr_DQM <= #tDLY regSEL[2];
    else if ((cState == c_wdata) && (clkCNT == 2))
            sdr_DQM <= #tDLY regSEL[3];
    else
            sdr_DQM <= #tDLY 0;

  always @(posedge sys_CLK or posedge sys_RESET)
    if (sys_RESET)
            regSysDX <= #tDLY 0;
    else if (cState == c_WRITEA)
            regSysDX <= #tDLY regSysD[7:0];
    else if ((cState == c_wdata) && (clkCNT == 0))
            regSysDX <= #tDLY regSysD[15:8];
    else if ((cState == c_wdata) && (clkCNT == 1))
            regSysDX <= #tDLY regSysD[23:16];
    else if ((cState == c_wdata) && (clkCNT == 2))
            regSysDX <= #tDLY regSysD[31:24];
    else
            regSysDX <= #tDLY 0;
  `else
    `ifdef SDR_DATA_SIZE_16
  always @(posedge sys_CLK or posedge sys_RESET)
    if (sys_RESET)
            sdr_DQM <= #tDLY 0;
    else if (cState == c_WRITEA)
            sdr_DQM <= #tDLY regSEL[1:0];
    else if ((cState == c_wdata) && (clkCNT == 0))
            sdr_DQM <= #tDLY regSEL[3:2];
    else
            sdr_DQM <= #tDLY 0;

  always @(posedge sys_CLK or posedge sys_RESET)
    if (sys_RESET)
            regSysDX <= #tDLY 0;
    else if (cState == c_WRITEA)
            regSysDX <= #tDLY regSysD[15:0];
    else if ((cState == c_wdata) && (clkCNT == 0))
            regSysDX <= #tDLY regSysD[31:16];
    else
            regSysDX <= #tDLY 0;

    `else
  always @(posedge sys_CLK or posedge sys_RESET)
    if (sys_RESET)
            sdr_DQM <= #tDLY 0;
    else if (cState == c_WRITEA)
            sdr_DQM <= #tDLY regSEL;
    else
            sdr_DQM <= #tDLY 0;

  always @(posedge sys_CLK or posedge sys_RESET)
    if (sys_RESET)
            regSysDX <= #tDLY 0;
    else if (cState == c_WRITEA)
            regSysDX <= #tDLY regSysD;
    else
            regSysDX <= #tDLY 0;
    `endif
  `endif
 `endif

wire [`SDR_DATA_WIDTH-1:0] write_sdr;
assign #tDLY write_sdr = (cState == c_wdata) || (cState == c_WRITEA) ? {`SDR_DATA_WIDTH{1'b0}} : {`SDR_DATA_WIDTH{1'b1}};


always @(posedge sys_CLK or posedge sys_RESET)
  if(sys_RESET)
    enableSdrDQ <= #tDLY {`SDR_DATA_WIDTH{1'b1}};
  else
    enableSdrDQ <= #tDLY write_sdr;

assign #tDLY regSysD = wb2sdr_q[63:32];
assign #tDLY regSEL = ~wb2sdr_q[67:64];

endmodule

