//   ==================================================================
//   >>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
//   ------------------------------------------------------------------
//   Copyright (c) 2006-2011 by Lattice Semiconductor Corporation
//   ALL RIGHTS RESERVED 
//   ------------------------------------------------------------------
//
//   IMPORTANT: THIS FILE IS AUTO-GENERATED BY THE LATTICEMICO SYSTEM.
//
//   Permission:
//
//      Lattice Semiconductor grants permission to use this code
//      pursuant to the terms of the Lattice Semiconductor Corporation
//      Open Source License Agreement.  
//
//   Disclaimer:
//
//      Lattice Semiconductor provides no warranty regarding the use or
//      functionality of this code. It is the user's responsibility to
//      verify the users design for consistency and functionality through
//      the use of formal verification methods.
//
//   --------------------------------------------------------------------
//
//                  Lattice Semiconductor Corporation
//                  5555 NE Moore Court
//                  Hillsboro, OR 97214
//                  U.S.A
//
//                  TEL: 1-800-Lattice (USA and Canada)
//                         503-286-8001 (other locations)
//
//                  web: http://www.latticesemi.com/
//                  email: techsupport@latticesemi.com
//
//   --------------------------------------------------------------------
//                         FILE DETAILS
// Project          : SDRAM Controller
// File             : sdr_fifo_intf.v
// Title            : SDRAM interface to the Wishbone bus 
// Dependencies     : sdr_par.v
// Revision History : 1.0: Initial Release
//                    3.0: Support for burst reads
//                    3.1: Support for burst writes
//                    3.2: (1) PLL used for SDRAM clock generation
//                         (2) IO registers are used for all SDRAM interface signals
//                    3.3: (1) PMI PLL updated to use correct primitive
//                         (2) Fix precharge bug for reads/writes
//                    3.4: (1) Fix for overflow bug for 32-bit burst reads
//                         (2) Fix for column addressing bug for 32-bit reads/writes
// =============================================================================


`define sdr_COMMAND  {sdr_CSn, sdr_RASn, sdr_CASn, sdr_WEn}

module sdr_fifo_intf(
                  //Sdram interface
                  sdr_DQ,        // sdr data
                  sdr_A,         // sdr address
                  sdr_BA,        // sdr bank address
                  sdr_CKE,       // sdr clock enable
                  sdr_CSn,       // sdr chip select
                  sdr_RASn,      // sdr row address
                  sdr_CASn,      // sdr column select
                  sdr_WEn,       // sdr write enable
                  sdr_DQM,       // sdr write data mask
                  S_STB_I,
                  read_tmn,
                  write_done,
                  //FIFOs interface at the side of SDRAM side
                  //Wishbone => SDRAM FIFO
                  wb2sdr_re,
                  wb2sdr_q,
                  wb2sdr_empty,
                  wb2sdr_almost_empty,
                  //SDRAM => Wishbone FIFO
                  sdr2wb_we,
                  sdr2wb_data,
                  sdr2wb_full,
                  sdr2wb_almost_full,
                  //System interface
                  sys_DLY_100US, // sdr power and clock stable for 100 us
                  //Global
                  sdr_clk,
                  RST_I
                  );

`include "sdr_par.v"
//Sdram interface
inout  [`SDR_DATA_WIDTH-1:0]  sdr_DQ;
output [`SDR_ADR_WIDTH-1:0]   sdr_A;
output [`SDR_BNK_WDTH-1:0]    sdr_BA;
output                        sdr_CKE;
output                        sdr_CSn;
output                        sdr_RASn;
output                        sdr_CASn;
output                        sdr_WEn;
output [`SDR_USER_DM-1:0]     sdr_DQM;
input             S_STB_I;
input             RST_I;
input     [68:0]  wb2sdr_q;
input             wb2sdr_empty;
input             wb2sdr_almost_empty;
input             sdr2wb_full;
input             sdr2wb_almost_full;
input             sys_DLY_100US;
input             sdr_clk;
output            read_tmn;
output            write_done;
output            wb2sdr_re;
output            sdr2wb_we;
output    [31:0]  sdr2wb_data;

// intermodule wires

wire [3:0]        iState;    // INIT_FSM state variables
wire [3:0]        cState;    // CMD_FSM state variables
wire [3:0]        cState_d;
wire [3:0]        clkCNT;
wire              rd_cmd;
wire              cross_page;
wire              write_done;
wire [`SDR_COL_WIDTH+`SDR_BNK_WDTH:0] rd_cmd_cnt;

sdr_ctrl U1 (
  .sys_CLK(sdr_clk),
  .sys_RESET(RST_I),
  .wb2sdr_re(wb2sdr_re),
  .wb2sdr_q(wb2sdr_q),
  .wb2sdr_empty(wb2sdr_empty),
  .wb2sdr_almost_empty(wb2sdr_almost_empty),
  .sdr2wb_full(sdr2wb_full),
  .sdr2wb_almost_full(sdr2wb_almost_full),
  .sys_DLY_100US(sys_DLY_100US),
  .iState(iState),
  .cState(cState),
  .cState_d(cState_d),
  .clkCNT(clkCNT),
  .S_STB_I(S_STB_I),
  .read_tmn(read_tmn),
  .rd_cmd(rd_cmd),
  .rd_cmd_cnt(rd_cmd_cnt),
  .cross_page(cross_page),
  .write_done(write_done)
);

sdr_sig U2 (
  .sys_CLK(sdr_clk),
  .sys_RESET(RST_I),
  .wb2sdr_q(wb2sdr_q),
  .iState(iState),
  .cState(cState),
  .cState_d(cState_d),
  .rd_cmd(rd_cmd),
  .cross_page(cross_page),
  .sdr_CKE(sdr_CKE),
  .sdr_CSn(sdr_CSn),
  .sdr_RASn(sdr_RASn),
  .sdr_CASn(sdr_CASn),
  .sdr_WEn(sdr_WEn),
  .sdr_BA(sdr_BA),
  .sdr_A(sdr_A)
);

sdr_data U3 (
  .sys_CLK(sdr_clk),
  .sys_RESET(RST_I),
  .wb2sdr_q(wb2sdr_q),
  .sdr2wb_we(sdr2wb_we),
  .sdr2wb_data(sdr2wb_data),
  .sdr2wb_full(sdr2wb_full),
  .cState(cState),
  .clkCNT(clkCNT),
  .rd_cmd_cnt(rd_cmd_cnt),
  .sdr_DQM(sdr_DQM),
  .sdr_DQ(sdr_DQ)
);


endmodule
