//   ==================================================================
//   >>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
//   ------------------------------------------------------------------
//   Copyright (c) 2006-2011 by Lattice Semiconductor Corporation
//   ALL RIGHTS RESERVED 
//   ------------------------------------------------------------------
//
//   IMPORTANT: THIS FILE IS AUTO-GENERATED BY THE LATTICEMICO SYSTEM.
//
//   Permission:
//
//      Lattice Semiconductor grants permission to use this code
//      pursuant to the terms of the Lattice Semiconductor Corporation
//      Open Source License Agreement.  
//
//   Disclaimer:
//
//      Lattice Semiconductor provides no warranty regarding the use or
//      functionality of this code. It is the user's responsibility to
//      verify the users design for consistency and functionality through
//      the use of formal verification methods.
//
//   --------------------------------------------------------------------
//
//                  Lattice Semiconductor Corporation
//                  5555 NE Moore Court
//                  Hillsboro, OR 97214
//                  U.S.A
//
//                  TEL: 1-800-Lattice (USA and Canada)
//                         503-286-8001 (other locations)
//
//                  web: http://www.latticesemi.com/
//                  email: techsupport@latticesemi.com
//
//   --------------------------------------------------------------------
//                         FILE DETAILS
// Project          : SDRAM Controller
// File             : sdr_par.v
// Title            : SDRAM parameter file
// Dependencies     : None
// Revision History : 1.0: Initial Release
//                    3.0: Support for burst reads
//                    3.1: Support for burst writes
//                    3.2: (1) PLL used for SDRAM clock generation
//                         (2) IO registers are used for all SDRAM interface signals
//                    3.3: (1) PMI PLL updated to use correct primitive
//                         (2) Fix precharge bug for reads/writes
//                    3.4: (1) Fix for overflow bug for 32-bit burst reads
//                         (2) Fix for column addressing bug for 32-bit reads/writes
// =============================================================================

// User modifiable parameters
/****************************
* Bus width setting
****************************/
`ifdef SDR_DATA_SIZE_32
`define SDR_DATA_WIDTH 32
`endif
`ifdef SDR_DATA_SIZE_16
`define SDR_DATA_WIDTH 16
`endif
`ifdef SDR_DATA_SIZE_8
`define SDR_DATA_WIDTH 8
`endif
`ifdef SDR_DATA_SIZE_4
`define SDR_DATA_WIDTH 4
`endif

/****************************
* Mode register setting
****************************/
// SDRAM mode register definition

// Write Burst Mode, 0: Programmed_Length, 1: Single_Access
`define   MR_Write_Burst_Mode   1'b0

// Operation Mode: Standard
`define   MR_Operation_Mode     2'b00

// CAS Latency, 3'b010: Latency_2,  3'b011: Latency_3
`define   MR_CAS_Latency        3'b011

// Burst Type, 1'b0: Sequential, 1'b1: Interleaved: 1'b1
`define   MR_Burst_Type         1'b0

// Burst Length
// 3'b000  burst length=1 for DATA_WIDTH 32
// 3'b001  burst length=2 for DATA_WIDTH 16
// 3'b010  burst length=4 for DATA_WIDTH 8
// 3'b011  burst length=8 for DATA_WIDTH 4
// 3'b111  burst length=full page
`ifdef SDR_DATA_SIZE_32
  `define   MR_Burst_Length       3'b000
`endif
`ifdef SDR_DATA_SIZE_16
  `define   MR_Burst_Length       3'b001
`endif
`ifdef SDR_DATA_SIZE_8
  `define   MR_Burst_Length       3'b010
`endif
`ifdef SDR_DATA_SIZE_4
  `define   MR_Burst_Length       3'b011
`endif


//----------------------------------------------------------------------------
`define SDR_ADR_WIDTH `SDR_ROW_WIDTH

`ifdef SDR_DATA_SIZE_4
`define SDR_USER_DM 1
`else
`define SDR_USER_DM `SDR_DATA_WIDTH/8
`endif

// tDAL needs to be satisfied before the next sdram ACTIVE command can
// be issued. State c_tDAL of CMD_FSM is created for this purpose.
// However, states c_idle, c_ACTIVE and c_tRCD need to be taken into
// account because ACTIVE command will not be issued until CMD_FSM
// switch from c_ACTIVE to c_tRCD. NUM_CLK_WAIT is the version after
// the adjustment.
parameter NUM_CLK_WAIT = (`SDR_TDAL < 4) ? 0 : `SDR_TDAL - 4;

parameter NUM_CLK_CL      = `MR_CAS_Latency;

`ifdef SDR_DATA_SIZE_32
parameter NUM_CLK_READ    = 1;
parameter NUM_CLK_WRITE   = 1;
`endif

`ifdef SDR_DATA_SIZE_16
parameter NUM_CLK_READ    = 2;
parameter NUM_CLK_WRITE   = 2;
`endif

`ifdef SDR_DATA_SIZE_8
parameter NUM_CLK_READ    = 4;
parameter NUM_CLK_WRITE   = 4;
`endif

`ifdef SDR_DATA_SIZE_4
parameter NUM_CLK_READ    = 8;
parameter NUM_CLK_WRITE   = 8;
`endif

//---------------------------------------------------------------------
// INIT_FSM state variable assignments (gray coded)
//

parameter i_NOP   = 4'b0000;
parameter i_PRE   = 4'b0001;
parameter i_tRP   = 4'b0010;
parameter i_AR1   = 4'b0011;
parameter i_tRFC1 = 4'b0100;
parameter i_AR2   = 4'b0101;
parameter i_tRFC2 = 4'b0110;
parameter i_MRS   = 4'b0111;
parameter i_tMRD  = 4'b1000;
parameter i_ready = 4'b1001;

//---------------------------------------------------------------------
// CMD_FSM state variable assignments (gray coded)
//

parameter c_idle   = 4'b0000;
parameter c_tRCD   = 4'b0001;
parameter c_cl     = 4'b0010;
parameter c_rdata  = 4'b0011;
parameter c_wdata  = 4'b0100;
parameter c_tRFC   = 4'b0101;
parameter c_tDAL   = 4'b0110;
parameter c_ACTIVE = 4'b1000;
parameter c_READA  = 4'b1111;
parameter c_WRITEA = 4'b1110;
parameter c_AR     = 4'b1011;
parameter c_ReWait = 4'b1100;
parameter c_PRECH  = 4'b1001;
parameter c_tRP    = 4'b1010;
//---------------------------------------------------------------------
// SDRAM commands (sdr_CSn, sdr_RASn, sdr_CASn, sdr_WEn)
//

parameter INHIBIT            = 4'b1111;
parameter NOP                = 4'b0111;
parameter ACTIVE             = 4'b0011;
parameter READ               = 4'b0101;
parameter WRITE              = 4'b0100;
parameter BURST_TERMINATE    = 4'b0110;
parameter PRECHARGE          = 4'b0010;
parameter AUTO_REFRESH       = 4'b0001;
parameter LOAD_MODE_REGISTER = 4'b0000;


parameter tDLY = 2; // 2ns delay for simulation purpose
