//   ==================================================================
//   >>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
//   ------------------------------------------------------------------
//   Copyright (c) 2006-2011 by Lattice Semiconductor Corporation
//   ALL RIGHTS RESERVED 
//   ------------------------------------------------------------------
//
//   IMPORTANT: THIS FILE IS AUTO-GENERATED BY THE LATTICEMICO SYSTEM.
//
//   Permission:
//
//      Lattice Semiconductor grants permission to use this code
//      pursuant to the terms of the Lattice Semiconductor Corporation
//      Open Source License Agreement.  
//
//   Disclaimer:
//
//      Lattice Semiconductor provides no warranty regarding the use or
//      functionality of this code. It is the user's responsibility to
//      verify the users design for consistency and functionality through
//      the use of formal verification methods.
//
//   --------------------------------------------------------------------
//
//                  Lattice Semiconductor Corporation
//                  5555 NE Moore Court
//                  Hillsboro, OR 97214
//                  U.S.A
//
//                  TEL: 1-800-Lattice (USA and Canada)
//                         503-286-8001 (other locations)
//
//                  web: http://www.latticesemi.com/
//                  email: techsupport@latticesemi.com
//
//   --------------------------------------------------------------------
//                         FILE DETAILS
// Project          : SDRAM Controller
// File             : sdr_sig.v
// Title            : SDRAM control signal generation logic
// Dependencies     : sdr_par.v
// Revision History : 1.0: Initial Release
//                    3.0: Support for burst reads
//                    3.1: Support for burst writes
//                    3.2: (1) PLL used for SDRAM clock generation
//                         (2) IO registers are used for all SDRAM interface signals
//                    3.3: (1) PMI PLL updated to use correct primitive
//                         (2) Fix precharge bug for reads/writes
//                    3.4: (1) Fix for overflow bug for 32-bit burst reads
//                         (2) Fix for column addressing bug for 32-bit reads/writes
// =============================================================================


`define sdr_COMMAND  {sdr_CSn, sdr_RASn, sdr_CASn, sdr_WEn}

module sdr_sig(
  sys_CLK,
  sys_RESET,
  wb2sdr_q,
  iState,
  cState,
  cState_d,
  rd_cmd,
  cross_page,
  sdr_CKE,    // sdr clock enable
  sdr_CSn,    // sdr chip select
  sdr_RASn,   // sdr row address
  sdr_CASn,   // sdr column select
  sdr_WEn,    // sdr write enable
  sdr_BA,     // sdr bank address
  sdr_A       // sdr address
);

`include "sdr_par.v"

`ifdef SDR_DATA_SIZE_32
  parameter LOW_0_CA = 0;
`else
  `ifdef SDR_DATA_SIZE_16
  parameter LOW_0_CA = 1;
  `else
    `ifdef SDR_DATA_SIZE_8
     parameter LOW_0_CA = 2;
    `else
     parameter LOW_0_CA = 3;
    `endif
  `endif
`endif

parameter CA_LSB = 2;
parameter CA_MSB = `SDR_COL_WIDTH+CA_LSB-1-LOW_0_CA;
parameter BA_LSB = CA_MSB+1;
parameter BA_MSB = `SDR_BNK_WDTH+BA_LSB-1;
parameter RA_LSB = BA_MSB+1;
parameter RA_MSB = `SDR_ROW_WIDTH+RA_LSB-1;

//---------------------------------------------------------------------
// inputs
//
input                     sys_CLK;
input                     sys_RESET;
input [68:0]              wb2sdr_q;
input [3:0]               iState;
input [3:0]               cState;
input [3:0]               cState_d;
input                     rd_cmd;
output                    cross_page;

//---------------------------------------------------------------------
// outputs
//
output                      sdr_CKE /*synthesis useioff=1 */;
output                      sdr_CSn /*synthesis useioff=1 */;
output                      sdr_RASn/*synthesis useioff=1 */;
output                      sdr_CASn/*synthesis useioff=1 */;
output                      sdr_WEn /*synthesis useioff=1 */;
output [`SDR_BNK_WDTH-1:0]  sdr_BA  /*synthesis useioff=1 */;
output [`SDR_ADR_WIDTH-1:0] sdr_A   /*synthesis useioff=1 */;

reg                         sdr_CKE /* synthesis syn_preserve=1 */;
reg                         sdr_CSn /* synthesis syn_preserve=1 */;
reg                         sdr_RASn/* synthesis syn_preserve=1 */;
reg                         sdr_CASn/* synthesis syn_preserve=1 */;
reg                         sdr_WEn /* synthesis syn_preserve=1 */;
reg [`SDR_BNK_WDTH-1:0]     sdr_BA  /* synthesis syn_preserve=1 */;
reg [`SDR_ADR_WIDTH-1:0]    sdr_A   /* synthesis syn_preserve=1 */;
   
reg  [31:0]      sys_A;
wire [31:0]      next_adr;
reg [`SDR_ADR_WIDTH+`SDR_BNK_WDTH-1:0] row_addr;
wire             cross_page;
reg              cross_page_dly;
wire [`SDR_ADR_WIDTH-1:0] col_addr;

//assign col_addr = {sys_A[CA_MSB:CA_LSB],{LOW_0_CA{1'b0}}};
   generate
      if (LOW_0_CA == 0) begin
	 assign col_addr = {4'b0000,sys_A[CA_MSB:CA_LSB]};
      end
      else begin
	 assign col_addr = {sys_A[CA_MSB:CA_LSB],{LOW_0_CA{1'b0}}};
      end 
   endgenerate
   
assign #tDLY next_adr = sys_A+4;

always @(posedge sys_CLK or posedge sys_RESET)
   if (sys_RESET) begin
      sys_A <= #tDLY 0;
      row_addr <= #tDLY 0;
      cross_page_dly <= #tDLY 1'b0;
   end else begin
      cross_page_dly <= #tDLY cross_page;
      if (cState_d == c_idle && cState == c_ReWait) begin
         sys_A    <= #tDLY wb2sdr_q[31:0];
         row_addr <= #tDLY wb2sdr_q[RA_MSB:BA_LSB];
      end else if ((cState == c_READA && rd_cmd == 1'b1) || cState == c_WRITEA) begin
         sys_A    <= #tDLY sys_A + 4;
         row_addr <= #tDLY next_adr[RA_MSB:BA_LSB];
      end
   end
assign cross_page = (cState == c_READA || cState == c_WRITEA) ? ((next_adr[RA_MSB:BA_LSB] == row_addr) ? 1'b0 : 1'b1) :
                    cState == c_ACTIVE ? 1'b0 : cross_page_dly;

//---------------------------------------------------------------------
// SDR SDRAM Control Singals
//
always @(posedge sys_CLK or posedge sys_RESET)
  if (sys_RESET) begin
    `sdr_COMMAND <= #tDLY INHIBIT;
    sdr_CKE <= #tDLY 0;
    sdr_BA  <= #tDLY {`SDR_BNK_WDTH{1'b1}};
    sdr_A   <= #tDLY {`SDR_ADR_WIDTH{1'b1}};
  end else
    case (iState)
      i_tRP,
      i_tRFC1,
      i_tRFC2,
      i_tMRD,
      i_NOP: begin
               `sdr_COMMAND <= #tDLY NOP;
               sdr_CKE <= #tDLY 1;
               sdr_BA  <= #tDLY {`SDR_BNK_WDTH{1'b1}};
               sdr_A   <= #tDLY {`SDR_ADR_WIDTH{1'b1}};
             end
      i_PRE: begin
               `sdr_COMMAND <= #tDLY PRECHARGE;
               sdr_CKE <= #tDLY 1;
               sdr_BA  <= #tDLY {`SDR_BNK_WDTH{1'b1}};
               sdr_A   <= #tDLY {`SDR_ADR_WIDTH{1'b1}};
             end
      i_AR1,
      i_AR2: begin
               `sdr_COMMAND <= #tDLY AUTO_REFRESH;
               sdr_CKE <= #tDLY 1;
               sdr_BA  <= #tDLY {`SDR_BNK_WDTH{1'b1}};
               sdr_A   <= #tDLY {`SDR_ADR_WIDTH{1'b1}};
             end
      i_MRS: begin
               `sdr_COMMAND <= #tDLY LOAD_MODE_REGISTER;
               sdr_CKE <= #tDLY 1;
               sdr_BA  <= #tDLY {`SDR_BNK_WDTH{1'b0}};
               sdr_A   <= #tDLY {
                            2'b00,
                            `MR_Write_Burst_Mode,
                            `MR_Operation_Mode,
                            `MR_CAS_Latency,
                            `MR_Burst_Type,
                            `MR_Burst_Length
                          };
             end
      i_ready:
             case (cState)
               c_idle,
               c_tRCD,
               c_tRFC,
               c_cl,
               c_rdata,
               c_tRP,
               c_wdata:  begin
                           `sdr_COMMAND <= #tDLY NOP;
                           sdr_CKE <= #tDLY 1;
                           sdr_BA  <= #tDLY {`SDR_BNK_WDTH{1'b1}};
                           sdr_A   <= #tDLY {`SDR_ADR_WIDTH{1'b1}};
                         end
               c_ACTIVE: begin
                           `sdr_COMMAND <= #tDLY ACTIVE;
                           sdr_CKE <= #tDLY 1;
                           sdr_BA  <= #tDLY sys_A[BA_MSB:BA_LSB];//bank
                           sdr_A   <= #tDLY sys_A[RA_MSB:RA_LSB];//row
                         end
               c_READA:  begin
                           `sdr_COMMAND <= #tDLY rd_cmd ? READ : NOP;
                           sdr_CKE <= #tDLY 1;
                           sdr_BA  <= #tDLY sys_A[BA_MSB:BA_LSB];//bank
                           sdr_A   <= #tDLY {col_addr[`SDR_ADR_WIDTH-1:11],1'b0,col_addr[9:0]};
                           end
               c_PRECH:  begin
                           `sdr_COMMAND <= #tDLY PRECHARGE;
                           sdr_CKE <= #tDLY 1;
                           sdr_BA  <= #tDLY {`SDR_BNK_WDTH{1'b1}};
                           sdr_A   <= #tDLY {`SDR_ADR_WIDTH{1'b1}};
                           end
               c_WRITEA: begin
                           `sdr_COMMAND <= #tDLY WRITE;
                           sdr_CKE <= #tDLY 1;
                           sdr_BA  <= #tDLY sys_A[BA_MSB:BA_LSB];//bank
                           sdr_A   <= #tDLY {col_addr[`SDR_ADR_WIDTH-1:11],1'b0,col_addr[9:0]};
                         end
               c_AR:     begin
                           `sdr_COMMAND <= #tDLY AUTO_REFRESH;
                           sdr_CKE <= #tDLY 1;
                           sdr_BA  <= #tDLY {`SDR_BNK_WDTH{1'b1}};
                           sdr_A   <= #tDLY {`SDR_ADR_WIDTH{1'b1}};
                         end
               default:  begin
                           `sdr_COMMAND <= #tDLY NOP;
                           sdr_CKE <= #tDLY 1;
                           sdr_BA  <= #tDLY {`SDR_BNK_WDTH{1'b1}};
                           sdr_A   <= #tDLY {`SDR_ADR_WIDTH{1'b1}};
                         end
             endcase
      default:
             begin
               `sdr_COMMAND <= #tDLY NOP;
               sdr_CKE <= #tDLY 1;
               sdr_BA  <= #tDLY {`SDR_BNK_WDTH{1'b1}};
               sdr_A   <= #tDLY {`SDR_ADR_WIDTH{1'b1}};
             end
    endcase

endmodule

