//   ==================================================================
//   >>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
//   ------------------------------------------------------------------
//   Copyright (c) 2006-2011 by Lattice Semiconductor Corporation
//   ALL RIGHTS RESERVED 
//   ------------------------------------------------------------------
//
//   IMPORTANT: THIS FILE IS AUTO-GENERATED BY THE LATTICEMICO SYSTEM.
//
//   Permission:
//
//      Lattice Semiconductor grants permission to use this code
//      pursuant to the terms of the Lattice Semiconductor Corporation
//      Open Source License Agreement.  
//
//   Disclaimer:
//
//      Lattice Semiconductor provides no warranty regarding the use or
//      functionality of this code. It is the user's responsibility to
//      verify the users design for consistency and functionality through
//      the use of formal verification methods.
//
//   --------------------------------------------------------------------
//
//                  Lattice Semiconductor Corporation
//                  5555 NE Moore Court
//                  Hillsboro, OR 97214
//                  U.S.A
//
//                  TEL: 1-800-Lattice (USA and Canada)
//                         503-286-8001 (other locations)
//
//                  web: http://www.latticesemi.com/
//                  email: techsupport@latticesemi.com
//
//   --------------------------------------------------------------------
//                         FILE DETAILS
// Project          : SDRAM Controller
// File             : wb_fifo_intf.v
// Title            : Logic to control the read/wrtie to the interface FIFO
//                    from Wishbone
// Dependencies     : sdr_par.v
// Revision History : 1.0: Initial Release
//                    3.0: Support for burst reads
//                    3.1: Support for burst writes
//                    3.2: (1) PLL used for SDRAM clock generation
//                         (2) IO registers are used for all SDRAM interface signals
//                    3.3: (1) PMI PLL updated to use correct primitive
//                         (2) Fix precharge bug for reads/writes
//                    3.4: (1) Fix for overflow bug for 32-bit burst reads
//                         (2) Fix for column addressing bug for 32-bit reads/writes
// =============================================================================

module wb_fifo_intf(
                  //Wishbone interface
                  S_ADR_I,
                  S_SEL_I,
                  S_DAT_I,
                  S_DAT_O,
                  S_WE_I,
                  S_ACK_O,
                  S_ERR_O,
                  S_RTY_O,
                  S_CTI_I,
                  S_BTE_I,
                  S_LOCK_I,
                  S_CYC_I,
                  S_STB_I,
                  CLK_I,
                  //FIFOs interface at the side of Wishbone side
                  //Wishbone => SDRAM FIFO
                  wb2sdr_we,
                  wb2sdr_data,
                  wb2sdr_almost_full,
                  wb2sdr_empty,
                  write_done,
                  //SDRAM => Wishbone FIFO
                  sdr2wb_re,
                  sdr2wb_q,
                  sdr2wb_empty,
                  //Global
                  RST_I
                  );
`include "sdr_par.v"

//Wishbone interface
input     [31:0]  S_ADR_I;
input     [31:0]  S_DAT_I;
input     [3:0]   S_SEL_I;
input             S_WE_I;
input     [2:0]   S_CTI_I;
input     [1:0]   S_BTE_I;
input             S_LOCK_I;
input             S_CYC_I;
input             S_STB_I;
input             CLK_I;
input             RST_I;
output    [31:0]  S_DAT_O;
output            S_ACK_O;
output            S_ERR_O;
output            S_RTY_O;
input             wb2sdr_almost_full;
input             wb2sdr_empty;
input             write_done;
input     [31:0]  sdr2wb_q;
input             sdr2wb_empty;
output            wb2sdr_we;
output    [68:0]  wb2sdr_data;
output            sdr2wb_re;

wire      [31:0]  S_DAT_O;
wire              wb2sdr_we;
wire      [68:0]  wb2sdr_data;
wire              sdr2wb_re;
wire              wb_start;
reg               S_ACK_O;
reg       [1:0]   wb_status;
parameter WB_IDLE   = 2'b00;
parameter WB_WRITE  = 2'b01;
parameter WB_WRWAIT = 2'b10;
parameter WB_READ   = 2'b11;

assign    S_ERR_O = 1'b0;
assign    S_RTY_O = 1'b0;

// wb interface operation

assign #tDLY S_DAT_O   = sdr2wb_q;
assign #tDLY sdr2wb_re = ~sdr2wb_empty;
assign #tDLY wb_start  = S_CYC_I && S_STB_I && sdr2wb_empty;
assign #tDLY wb2sdr_we = S_WE_I == 1'b1       ? S_ACK_O & S_STB_I :
                         wb_status == WB_IDLE ? wb_start : 1'b0;
assign #tDLY wb2sdr_data = {S_WE_I,S_SEL_I,S_DAT_I,S_ADR_I};

always @(posedge RST_I or posedge CLK_I)
if(RST_I) begin
   wb_status   <= WB_IDLE;
   S_ACK_O     <= 1'b0;
end else
   case(wb_status)
   WB_IDLE:   if (wb_start) begin
                 S_ACK_O     <= #tDLY S_WE_I ? 1'b1 : 1'b0;
                 wb_status   <= #tDLY S_WE_I ? WB_WRITE : WB_READ;
              end else begin
                 S_ACK_O     <= #tDLY 1'b0;
                 wb_status   <= #tDLY WB_IDLE;
              end
   WB_WRITE:  if (!S_STB_I) begin
                 S_ACK_O    <= #tDLY 1'b0;
                 wb_status  <= #tDLY WB_WRWAIT;
              end else if (S_STB_I & S_ACK_O & (S_CTI_I != 3'b010)) begin
                 S_ACK_O    <= #tDLY 1'b0;
                 wb_status  <= #tDLY WB_WRWAIT;
              end else if(!wb2sdr_almost_full) begin
                 S_ACK_O     <= #tDLY 1'b1;
              end else begin
                 S_ACK_O    <= #tDLY 1'b0;
              end
   WB_WRWAIT: if (write_done)
                 wb_status  <= #tDLY WB_IDLE;
   WB_READ:   if (!S_STB_I) begin
                 S_ACK_O    <= #tDLY 1'b0;
                 wb_status  <= #tDLY WB_IDLE;
              end else if (S_STB_I & S_ACK_O & (S_CTI_I != 3'b010)) begin
                 S_ACK_O    <= #tDLY 1'b0;
                 wb_status  <= #tDLY WB_IDLE;
              end else begin
                 S_ACK_O    <= #tDLY sdr2wb_re;
              end
   default: wb_status <= #tDLY WB_IDLE;
   endcase

endmodule

