//   ==================================================================
//   >>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
//   ------------------------------------------------------------------
//   Copyright (c) 2006-2011 by Lattice Semiconductor Corporation
//   ALL RIGHTS RESERVED 
//   ------------------------------------------------------------------
//
//   IMPORTANT: THIS FILE IS AUTO-GENERATED BY THE LATTICEMICO SYSTEM.
//
//   Permission:
//
//      Lattice Semiconductor grants permission to use this code
//      pursuant to the terms of the Lattice Semiconductor Corporation
//      Open Source License Agreement.  
//
//   Disclaimer:
//
//      Lattice Semiconductor provides no warranty regarding the use or
//      functionality of this code. It is the user's responsibility to
//      verify the users design for consistency and functionality through
//      the use of formal verification methods.
//
//   --------------------------------------------------------------------
//
//                  Lattice Semiconductor Corporation
//                  5555 NE Moore Court
//                  Hillsboro, OR 97214
//                  U.S.A
//
//                  TEL: 1-800-Lattice (USA and Canada)
//                         503-286-8001 (other locations)
//
//                  web: http://www.latticesemi.com/
//                  email: techsupport@latticesemi.com
//
//   --------------------------------------------------------------------
//                         FILE DETAILS
// Project          : SDRAM Controller
// File             : wb_sdr_ctrl.v
// Title            : SDRAM controller's top file
// Dependencies     : sdr_par.v
// Revision History : 1.0: Initial Release
//                    3.0: Support for burst reads
//                    3.1: Support for burst writes
//                    3.2: (1) PLL used for SDRAM clock generation
//                         (2) IO registers are used for all SDRAM interface signals
//                    3.3: (1) PMI PLL updated to use correct primitive
//                         (2) Fix precharge bug for reads/writes
//                    3.4: (1) Fix for overflow bug for 32-bit burst reads
//                         (2) Fix for column addressing bug for 32-bit reads/writes
//                         (3) ECP3 family support
//                    3.5: SDRAM clock generation fixed for ECP3 silicon.
//					  3.6: Support added to distinguish between ECP3 -E and -EA
//						   silicon. Default is -EA silicon.
// =============================================================================

module wb_sdr_ctrl 
  #(parameter SYS_FREQ = 25.0,
    parameter SDRAM_FREQ = 100,
    parameter LATTICE_FAMILY = "ECP2",
    parameter LATTICE_DEVICE = "All")
   (
    //Wishbone interface
    S_ADR_I,
    S_SEL_I,
    S_DAT_I,
    S_DAT_O,
    S_WE_I,
    S_ACK_O,
    S_ERR_O,
    S_RTY_O,
    S_CTI_I,
    S_BTE_I,
    S_LOCK_I,
    S_CYC_I,
    S_STB_I,
    CLK_I,
    //Sdram interface
    sdr_DQ,        // sdr data
    sdr_A,         // sdr address
    sdr_BA,        // sdr bank address
    sdr_CKE,       // sdr clock enable
    sdr_CSn,       // sdr chip select
    sdr_RASn,      // sdr row address
    sdr_CASn,      // sdr column select
    sdr_WEn,       // sdr write enable
    sdr_DQM,       // sdr write data mask
    sdr_CLK,
    //Global
    RST_I
    );
   
`include "sdr_par.v"

//Wishbone interface
input     [31:0]  S_ADR_I;
input     [31:0]  S_DAT_I;
output    [31:0]  S_DAT_O;
input     [3:0]   S_SEL_I;
input             S_WE_I;
output            S_ACK_O;
output            S_ERR_O;
output            S_RTY_O;
input     [2:0]   S_CTI_I;
input     [1:0]   S_BTE_I;
input             S_LOCK_I;
input             S_CYC_I;
input             S_STB_I;
input             CLK_I;
input             RST_I;

output  [`SDR_ADR_WIDTH-1:0] sdr_A;
output  [`SDR_BNK_WDTH-1:0]  sdr_BA;
output                       sdr_CKE;
output                       sdr_CSn;
output                       sdr_RASn;
output                       sdr_CASn;
output                       sdr_WEn;
output  [`SDR_USER_DM-1:0]   sdr_DQM;
inout   [`SDR_DATA_WIDTH-1:0]sdr_DQ;
output                       sdr_CLK;

wire      [68:0]  wb2sdr_data;
wire              wb2sdr_we;
wire              wb2sdr_re;
wire      [68:0]  wb2sdr_q;
wire              wb2sdr_empty;
wire              wb2sdr_almost_empty;
wire              wb2sdr_full;
wire              wb2sdr_almost_full;

wire      [31:0]  sdr2wb_data;
wire              sdr2wb_we;
wire              sdr2wb_re;
wire      [31:0]  sdr2wb_q;
wire              sdr2wb_empty;
wire              sdr2wb_full;
reg               sys_DLY_100US;
reg   [15:0]      sys_dly_cnt;
wire              read_tmn;
wire              write_done;
wire              sdr_clk_c;
wire              sdr_clk_io;

	generate
		if (LATTICE_FAMILY == "MachXO2") begin
		pmi_pll_fp #(.pmi_freq_clki(SYS_FREQ),
			.pmi_freq_clkop(SDRAM_FREQ),
			.pmi_freq_clkos(SDRAM_FREQ),
			.pmi_family("XO2"),
			.pmi_phase_adj(90),
			.pmi_duty_cycle(50),
			.pmi_clkfb_source("CLKOP"),
			.pmi_fdel("OFF"),
			.pmi_fdel_val(0),
			.module_type("pmi_pll"))
		U1_pmi_pll (.CLKI(CLK_I),
			.CLKFB(sdr_clk_c),
			.RESET(RST_I),
			.CLKOP(sdr_clk_c),
			.CLKOS(sdr_clk_io),
			.CLKOK(),
			.CLKOK2(),
			.LOCK()
			);
		end
		else begin
		pmi_pll_fp #(.pmi_freq_clki(SYS_FREQ),
			.pmi_freq_clkop(SDRAM_FREQ),
			.pmi_freq_clkos(SDRAM_FREQ),
			.pmi_family(`LATTICE_FAMILY),
			.pmi_phase_adj(90),
			.pmi_duty_cycle(50),
			.pmi_clkfb_source("CLKOP"),
			.pmi_fdel("OFF"),
			.pmi_fdel_val(0),
			.module_type("pmi_pll"))
		U1_pmi_pll (.CLKI(CLK_I),
			.CLKFB(sdr_clk_c),
			.RESET(RST_I),
			.CLKOP(sdr_clk_c),
			.CLKOS(sdr_clk_io),
			.CLKOK(),
			.CLKOK2(),
			.LOCK()
			);
		end
   endgenerate
   
   generate
      if (LATTICE_FAMILY == "EC") begin
	 ODDRXB sdr_clk_inst
	   (
	    .CLK(sdr_clk_io),
	    .DA(1'b0),
	    .DB(1'b1),
	    .LSR(1'b0),
	    .Q(sdr_CLK));
      end
      else if (LATTICE_FAMILY == "ECP") begin
	 ODDRXB sdr_clk_inst
	   (
	    .CLK(sdr_clk_io),
	    .DA(1'b0),
	    .DB(1'b1),
	    .LSR(1'b0),
	    .Q(sdr_CLK));
      end
      else if (LATTICE_FAMILY == "ECP2") begin
	 ODDRXC sdram_clk_inst
	   (
	    .CLK(sdr_clk_io), 
	    .DA(1'b1), 
	    .DB(1'b0), 
	    .RST(1'b0), 
	    .Q(sdr_CLK));
      end
      else if (LATTICE_FAMILY == "ECP2M") begin
	 ODDRXC sdram_clk_inst
	   (
	    .CLK(sdr_clk_io), 
	    .DA(1'b1), 
	    .DB(1'b0), 
	    .RST(1'b0), 
	    .Q(sdr_CLK));
      end
      else if (LATTICE_FAMILY == "XP") begin
	 ODDRXB sdr_clk_inst
	   (
	    .CLK(sdr_clk_io),
	    .DA(1'b0),
	    .DB(1'b1),
	    .LSR(1'b0),
	    .Q(sdr_CLK));
      end
      else if (LATTICE_FAMILY == "XP2") begin
	 ODDRXC sdram_clk_inst
	   (
	    .CLK(sdr_clk_io), 
	    .DA(1'b1), 
	    .DB(1'b0), 
	    .RST(1'b0), 
	    .Q(sdr_CLK));
      end
      else if (LATTICE_FAMILY == "SC") begin
      ODDRXA sdram_clk_inst
	(
	 .CLK(sdr_clk_io), 
	 .DA(1'b1), 
	 .DB(1'b0), 
	 .RST(1'b0), 
	 .Q(sdr_CLK));
      end
      else if (LATTICE_FAMILY == "SCM") begin
	 ODDRXA sdram_clk_inst
	   (
	    .CLK(sdr_clk_io), 
	    .DA(1'b1), 
	    .DB(1'b0), 
	    .RST(1'b0), 
	    .Q(sdr_CLK));
      end
	  else if (LATTICE_FAMILY == "MachXO2") begin
	 ODDRXE sdram_clk_inst   //INPUTS: D0, D1, SCLK, RST OUTPUT : Q
	   (
	    .SCLK(sdr_clk_io), 
	    .D0(1'b1), 
	    .D1(1'b0), 
	    .RST(1'b0), 
	    .Q(sdr_CLK));		  
		end
		else if (LATTICE_FAMILY == "ECP3") begin
	 if ((LATTICE_DEVICE == "LFE3-70E") || (LATTICE_DEVICE == "LFE3-95E")) begin
	    DQSBUFG U1_CLK_BUFG 
	      (
	       .SCLK (sdr_clk_io), 
	       .DDRCLKPOL (ddrclkpol_clk_io), 
	       .DQCLK1(dqclk1_clk)); 
	    
	    ODDRXD U1_ODDR_CLK 
	      (
	       .DA(1'b1),
	       .DB(1'b0),
	       .SCLK(sdr_clk_io),
	       .DQCLK1(dqclk1_clk),
	       .Q(sdr_CLK));
		   end
	 else begin
	    ODDRXD1 sdram_clk_inst 
	      (
	       .SCLK(sdr_clk_io),
	       .DA(1'b1),
	       .DB(1'b0),
	       .Q(sdr_CLK)) /*synthesis ODDRAPPS = "SCLK_ALIGNED" */;
	 end
      end
   endgenerate
   
always @(posedge sdr_clk_c or posedge RST_I)
if(RST_I) begin
   sys_DLY_100US <= 1'b0;
   sys_dly_cnt   <= `T100US; end
else begin
   if (sys_dly_cnt == 10'h0) begin
      sys_DLY_100US <= 1'b1;
      sys_dly_cnt <= sys_dly_cnt; end
   else begin
      sys_DLY_100US <= 1'b0;
      sys_dly_cnt <= sys_dly_cnt - 1;end
end

parameter READ_FIFO_DEPTH = 16;
parameter READ_FIFO_ALMOST_FULL = READ_FIFO_DEPTH-NUM_CLK_CL/NUM_CLK_READ-4;
parameter WRITE_FIFO_DEPTH = 16;
parameter WRITE_FIFO_ALMOST_EMPTY = 1;

pmi_fifo_dc #(
    .pmi_data_width_w      (69),                     
    .pmi_data_width_r      (69),                     
    .pmi_data_depth_w      (WRITE_FIFO_DEPTH),       
    .pmi_data_depth_r      (WRITE_FIFO_DEPTH),       
    .pmi_full_flag         (WRITE_FIFO_DEPTH),       
    .pmi_empty_flag        (0),                      
    .pmi_almost_full_flag  (WRITE_FIFO_DEPTH-1),     
    .pmi_almost_empty_flag (WRITE_FIFO_ALMOST_EMPTY),
    .pmi_regmode           ("noreg"),                
    .pmi_resetmode         ("async"),                
    .pmi_family            (`LATTICE_FAMILY),        
    .module_type           ("pmi_fifo_dc"),          
    .pmi_implementation    ("LUT")) 
   fifo_wb2sdr
   (.Data        (wb2sdr_data ),
    .WrClock     (CLK_I       ),
    .RdClock     (sdr_clk_c ),
    .WrEn        (wb2sdr_we   ),
    .RdEn        (wb2sdr_re   ),
    .Reset       (RST_I       ),
    .RPReset     (RST_I       ),
    .Q           (wb2sdr_q    ),
    .Empty       (wb2sdr_empty),
    .Full        (wb2sdr_full ),
    .AlmostEmpty (wb2sdr_almost_empty),
    .AlmostFull  (wb2sdr_almost_full));

pmi_fifo_dc #(
    .pmi_data_width_w      (32),                  
    .pmi_data_width_r      (32),                   
    .pmi_data_depth_w      (READ_FIFO_DEPTH),      
    .pmi_data_depth_r      (READ_FIFO_DEPTH),      
    .pmi_full_flag         (READ_FIFO_DEPTH),      
    .pmi_empty_flag        (0),                    
    .pmi_almost_full_flag  (READ_FIFO_ALMOST_FULL),
    .pmi_almost_empty_flag (1),                    
    .pmi_regmode           ("noreg"),              
    .pmi_resetmode         ("async"),              
    .pmi_family            (`LATTICE_FAMILY),      
    .module_type           ("pmi_fifo_dc"),        
    .pmi_implementation    ("LUT")) 
   fifo_sdr2wb
   (.Data        (sdr2wb_data ),
    .WrClock     (sdr_clk_c ),
    .RdClock     (CLK_I       ),
    .WrEn        (sdr2wb_we   ),
    .RdEn        (sdr2wb_re   ),
    .Reset       (read_tmn    ),
    .RPReset     (RST_I       ),
    .Q           (sdr2wb_q    ),
    .Empty       (sdr2wb_empty),
    .Full        (sdr2wb_full ),
    .AlmostEmpty (            ),
    .AlmostFull  (sdr2wb_almost_full));


wb_fifo_intf wb_fifo_intf_uut(
    .S_ADR_I     (S_ADR_I     ),
    .S_SEL_I     (S_SEL_I     ),
    .S_DAT_I     (S_DAT_I     ),
    .S_DAT_O     (S_DAT_O     ),
    .S_WE_I      (S_WE_I      ),
    .S_ACK_O     (S_ACK_O     ),
    .S_ERR_O     (S_ERR_O     ),
    .S_RTY_O     (S_RTY_O     ),
    .S_CTI_I     (S_CTI_I     ),
    .S_BTE_I     (S_BTE_I     ),
    .S_LOCK_I    (S_LOCK_I    ),
    .S_CYC_I     (S_CYC_I     ),
    .S_STB_I     (S_STB_I     ),
    .CLK_I       (CLK_I       ),

    .wb2sdr_we   (wb2sdr_we   ),
    .wb2sdr_data (wb2sdr_data ),
    .wb2sdr_almost_full (wb2sdr_almost_full ),
    .wb2sdr_empty(wb2sdr_empty),
    .write_done  (write_done  ),

    .sdr2wb_re   (sdr2wb_re   ),
    .sdr2wb_q    (sdr2wb_q    ),
    .sdr2wb_empty(sdr2wb_empty),

    .RST_I       (RST_I       )
    );

sdr_fifo_intf sdr_fifo_intf_uut(
    .sdr_DQ      (sdr_DQ       ),
    .sdr_A       (sdr_A        ),
    .sdr_BA      (sdr_BA       ),
    .sdr_CKE     (sdr_CKE      ),
    .sdr_CSn     (sdr_CSn      ),
    .sdr_RASn    (sdr_RASn     ),
    .sdr_CASn    (sdr_CASn     ),
    .sdr_WEn     (sdr_WEn      ),
    .sdr_DQM     (sdr_DQM      ),
    .S_STB_I     (S_STB_I      ),
    .read_tmn    (read_tmn     ),
    .write_done  (write_done   ),

    .sdr2wb_we   (sdr2wb_we   ),
    .sdr2wb_data (sdr2wb_data ),
    .sdr2wb_full (sdr2wb_full ),
    .sdr2wb_almost_full(sdr2wb_almost_full),

    .wb2sdr_re   (wb2sdr_re   ),
    .wb2sdr_q    (wb2sdr_q    ),
    .wb2sdr_empty(wb2sdr_empty),
    .wb2sdr_almost_empty(wb2sdr_almost_empty),
    .sys_DLY_100US(sys_DLY_100US),
    .sdr_clk     (sdr_clk_c ),
    .RST_I       (RST_I       )
    );

endmodule


